! gtool_file.f90 - gtool4 file interface
! Copyright (C) by TOYODA Eizi, 2000.  All rights reserved.
! vi: set sw=4 ts=8:

module gtool_file 

    use netcdf_file
    use netcdf_attribute
    use dc_string
    implicit none

    type GT_FILE
	type(NC_FILE):: ncfile
    end type

    interface Create;  module procedure GtoolFileCreate;  end interface
    interface Open;  module procedure GtoolFileOpen;  end interface
    interface Close;  module procedure GtoolFileClose;  end interface

    interface operator(.error.)
	module procedure GtoolFileError
    end interface

    interface operator(.netcdf.)
	module procedure GtoolFileNcpart
    end interface

    interface assignment(=)
	module procedure LetGtoolFileNetcdfFile
	module procedure LetNetcdfFileGtoolFile
    end interface

contains

    subroutine LetGtoolFileNetcdfFile(ncfile, gtfile)
	type(NC_FILE), intent(out):: ncfile
	type(GT_FILE), intent(in):: gtfile
    continue
	ncfile = gtfile%ncfile
    end subroutine

    subroutine LetNetcdfFileGtoolFile(gtfile, ncfile)
	type(GT_FILE), intent(out):: gtfile
	type(NC_FILE), intent(in):: ncfile
    continue
	gtfile%ncfile = ncfile
    end subroutine

    !
    ! --- Create/Open/Close ---
    !

    subroutine GtoolFileCreate(file, filename, overwrite, fail)
	type(GT_FILE), intent(inout):: file
	type(VARYING_STRING), intent(in):: filename
	logical, intent(in), optional:: overwrite
	logical, intent(out), optional:: fail
	type(NC_ATTRIBUTE):: attr
    continue
	call Create(file%ncfile, char(filename), overwrite=overwrite, fail=fail)
	attr = Attribute(file%ncfile, 'Conventions')
	attr = 'http://www.gfd-dennou.org/arch/gtool4/conventions/'
	call Dispose(attr)
	attr = Attribute(file%ncfile, 'gt_version')
	attr = '4.0beta9'
	call Dispose(attr)
    end subroutine

    subroutine GtoolFileOpen(file, filename, writable, fail)
	type(GT_FILE), intent(inout):: file
	type(VARYING_STRING), intent(in):: filename
	logical, intent(in), optional:: writable
	logical, intent(out), optional:: fail
    continue
	call Open(file%ncfile, char(filename), writable=writable, fail=fail)
    end subroutine

    subroutine GtoolFileClose(file, fail)
	type(GT_FILE), intent(inout):: file
	logical, optional, intent(out):: fail
	type(NC_ATTRIBUTE):: attr
	logical:: d_fail
    continue
	if (.writable.(file%ncfile)) then
	    attr = Attribute(file%ncfile, 'title')
	    if (.not. .exists. attr) then
		call DefineMode(file%ncfile, fail=d_fail)
		if (.not. d_fail) &
		    & attr = '(title not given: written by gtool4/Fortran90)'
	    endif
	endif
	call Close(file%ncfile, fail=fail)
    end subroutine

    logical function GtoolFileError(file) result(result)
	type(GT_FILE), intent(in):: file
    continue
	result = .error. file%ncfile
    end function

    type(NC_FILE) function GtoolFileNcpart(file) result(result)
	type(GT_FILE), intent(in):: file
    continue
	result = file%ncfile
    end function

end module
